%{
#include <stdio.h>
#include <stdlib.h>
#include "ael_structs.h"
extern void reset_parencount(void);
extern void reset_semicount(void);
#define YYLEX_PARAM scanner
#define YYPARSE_PARAM scanner
extern int yylineno;
void yyerror(/* void *locp,*/ char const *s);
%}


%union {
	char *str;

}

%{

%}

 
%token KW_CONTEXT LC RC LP RP SEMI EQ COMMA COLON AMPER BAR
%token KW_MACRO KW_GLOBALS KW_IGNOREPAT KW_SWITCH KW_IF KW_ELSE 
%token EXTENMARK KW_GOTO KW_RETURN KW_BREAK KW_CONTINUE
%token KW_FOR KW_WHILE KW_CASE KW_DEFAULT KW_CATCH KW_SWITCHES KW_ESWITCHES
%token KW_INCLUDES

%token <str> word

%locations
%pure-parser
%error-verbose

%%

file : objects
	;

objects : object
	| objects object;
	;

object : context
	| macro
	| globals
	;

context : KW_CONTEXT word LC elements RC SEMI
	| KW_CONTEXT word LC RC SEMI /* empty context OK */
	;

macro : KW_MACRO word LP arglist RP LC macro_statements RC SEMI
	| KW_MACRO word LP arglist RP LC  RC SEMI
	| KW_MACRO word LP RP LC macro_statements RC SEMI
	| KW_MACRO word LP RP LC  RC SEMI
	;

globals : KW_GLOBALS LC global_statements RC SEMI
	| KW_GLOBALS LC RC SEMI /* empty global is OK */
	;

global_statements : global_statement
	| global_statements global_statement
	;

global_statement : word EQ {reset_semicount();}  word SEMI
	;

arglist : word
	| arglist COMMA word
	;

elements : element
	| elements element
	;

element : extension
	| includes
	| switches
	| eswitches
	| ignorepat
	| word EQ word SEMI;
	;

ignorepat : KW_IGNOREPAT EXTENMARK word SEMI
	;

extension : word EXTENMARK statement
	;

statements : statement
	| statements statement
	;

if_head : KW_IF LP {reset_parencount();} word_list RP
	;

word_list : word
	| word word
	;

switch_head : KW_SWITCH LP {reset_parencount();} word RP  LC
	;

if_statement : word EQ {reset_semicount();} word SEMI
	| KW_GOTO target SEMI
	| word COLON
	| KW_FOR LP {reset_semicount();} word SEMI  {reset_semicount();} word SEMI {reset_parencount();} word RP statement
	| KW_WHILE LP {reset_parencount();} word RP statement
	| switch_head RC SEMI /* empty list OK */
	| switch_head case_statements RC SEMI
	| AMPER macro_call SEMI
	| application_call SEMI
	| KW_BREAK SEMI
	| KW_RETURN SEMI
	| KW_CONTINUE SEMI
	| if_head LC statements RC SEMI
	| if_head if_statement
	| if_head LC statements RC KW_ELSE statement
	;

statement : LC statements RC SEMI
	| word EQ {reset_semicount();} word SEMI
	| KW_GOTO target SEMI
	| word COLON
	| KW_FOR LP {reset_semicount();} word SEMI  {reset_semicount();} word SEMI {reset_parencount();} word RP statement
	| KW_WHILE LP {reset_parencount();} word RP statement
	| switch_head RC SEMI /* empty list OK */
	| switch_head case_statements RC SEMI
	| AMPER macro_call SEMI
	| application_call SEMI
	| KW_BREAK SEMI
	| KW_RETURN SEMI
	| KW_CONTINUE SEMI
	| if_head LC statements RC SEMI
	| if_head if_statement
	| if_head LC statements RC KW_ELSE statement
	;

target : word
	| word BAR word
	| word BAR word BAR word
	;

macro_call : word LP {reset_argcount();} eval_arglist RP
	| word LP RP
	;

application_call_head: word {reset_argcount();} LP 

application_call : application_call_head eval_arglist RP
	| application_call_head RP
	;

eval_arglist :  word_list
	| eval_arglist COMMA  word
	;

case_statements: case_statement
	| case_statements case_statement
	;

case_statement: KW_CASE word COLON statements
	| KW_DEFAULT COLON statements
	;

macro_statements: macro_statement
	| macro_statements macro_statement
	;

macro_statement : statement
	| KW_CATCH word LC statements RC SEMI
	;

switches : KW_SWITCHES LC switchlist RC SEMI
	| KW_SWITCHES LC RC SEMI /* empty switch list OK */
	;

eswitches : KW_ESWITCHES LC switchlist RC SEMI
	| KW_ESWITCHES LC  RC SEMI /* empty switch list OK */
	;

switchlist : word SEMI
	| switchlist word SEMI
	;

includes : KW_INCLUDES LC switchlist RC SEMI
	;


%%

void yyerror(/* YYLTYPE *locp,*/ char const *s)
{
	extern int my_lineno;
	fprintf(stderr,"line %d: error: %s\n", my_lineno, s);
#ifdef LATER
	fprintf(stderr,"line %d, col %d thru line %d, col %d:  %s\n", 
		locp->first_line, locp->first_column, 
		locp->last_line, locp->last_column, s);
#endif

}

